/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2011 Klaus Spanderen

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.
*/

#include <sandbox.hpp>
#include <ql/errors.hpp>

#include <fstream>
#include <sstream>

Sandbox::Sandbox() {
    JavaVMInitArgs vm_args;
    JavaVMOption options[2];
    
    char optionString[1024];
    std::strcpy(optionString, "-Djava.class.path=");
    options[0].optionString 
        = std::strncat(optionString, getenv("CLASSPATH"), 
                       sizeof(optionString)-std::strlen(optionString));
    
    options[1].optionString = strdup("-Xrs");
    vm_args.version = JNI_VERSION_1_6; 
    vm_args.nOptions = 1;
    vm_args.options = options;
    vm_args.ignoreUnrecognized = 0;
    
    JNI_CreateJavaVM(&jvm_, (void**)&env_, &vm_args);

    stringClass_ = env_->FindClass("java/lang/String");
    scalaBootLoaderClass_ = env_->FindClass("ScalaBootLoader");
    
    scalaBootLoader_ = env_->NewObject(scalaBootLoaderClass_, 
                                       env_->GetMethodID(scalaBootLoaderClass_,
                                                         "<init>","()V"));
    compileMethodID_ = env_->GetMethodID(scalaBootLoaderClass_, "compile",
                                         "([Ljava/lang/String;)V");
    payoffMethodID_  = env_->GetMethodID(scalaBootLoaderClass_, "payoff",
                                        "([D)[D");
}

Sandbox::~Sandbox() {
    env_->CallVoidMethod(scalaBootLoader_, 
                         env_->GetMethodID(scalaBootLoaderClass_, 
                                           "shutdown","()V"));
    if (env_->ExceptionOccurred()) {
        env_->ExceptionDescribe();
    }
    
    env_->DeleteLocalRef(stringClass_ ); 
    env_->DeleteLocalRef(scalaBootLoader_); 

    jvm_->DestroyJavaVM();
}

void Sandbox::compile(const std::string& fileName) const {
	std::ifstream file(fileName.c_str());
    QL_REQUIRE(file, "File not found");

	std::stringstream buffer;
	buffer << file.rdbuf();
	file.close();

	jstring jstr = env_->NewStringUTF(buffer.str().c_str());

	jobjectArray args = env_->NewObjectArray(1, stringClass_, jstr);
	env_->CallVoidMethod(scalaBootLoader_, compileMethodID_, args);
	env_->ReleaseStringUTFChars(jstr, 
                                env_->GetStringUTFChars(jstr, JNI_FALSE));
	env_->DeleteLocalRef(jstr); 
}

const Disposable<std::vector<Real> > 
Sandbox::payoff(const std::vector<Real>& path) const {
    const Size n(path.size());
	jdoubleArray x = env_->NewDoubleArray(n);
	env_->SetDoubleArrayRegion(x, 0, n, &path[0]);     

    jdoubleArray y = reinterpret_cast<jdoubleArray>(
        env_->CallObjectMethod(scalaBootLoader_, payoffMethodID_, x));
	env_->DeleteLocalRef(x);

    const Size m(env_->GetArrayLength(y));
    std::vector<Real> retVal(m);

    env_->GetDoubleArrayRegion(y, 0, m, &retVal[0]);
	env_->DeleteLocalRef(y);
    return retVal;
}

